@@ -1,8 +1,7 @@
 # gemato: OpenPGP signature support tests
-# (c) 2017-2023 Michał Górny
+# (c) 2017-2025 Michał Górny
 # SPDX-License-Identifier: GPL-2.0-or-later
 
-import base64
 import contextlib
 import datetime
 import io
@@ -10,8 +9,12 @@ import logging
 import os
 import shlex
 import signal
+import subprocess
 import tempfile
 
+from functools import cache
+from pathlib import Path
+
 import pytest
 
 import gemato.cli
@@ -24,6 +27,7 @@ from gemato.exceptions import (
     OpenPGPExpiredKeyFailure,
     OpenPGPRevokedKeyFailure,
     OpenPGPKeyImportError,
+    OpenPGPKeyListingError,
     OpenPGPKeyRefreshError,
     OpenPGPRuntimeError,
     OpenPGPUntrustedSigFailure,
@@ -31,6 +35,7 @@ from gemato.exceptions import (
     )
 from gemato.manifest import ManifestFile
 from gemato.openpgp import (
+    GNUPG,
     SystemGPGEnvironment,
     IsolatedGPGEnvironment,
     PGPyEnvironment,
@@ -41,225 +46,175 @@ from gemato.openpgp import (
     )
 from gemato.recursiveloader import ManifestRecursiveLoader
 
-from tests.keydata import (
-    PRIVATE_KEY_ID, KEY_FINGERPRINT, OTHER_KEY_FINGERPRINT, VALID_PUBLIC_KEY,
-    VALID_KEY_NOEMAIL, VALID_KEY_NONUTF, COMBINED_PUBLIC_KEYS,
-    VALID_KEY_SUBKEY, PRIVATE_KEY, EXPIRED_PUBLIC_KEY, REVOKED_PUBLIC_KEY,
-    OTHER_VALID_PUBLIC_KEY, UNSIGNED_PUBLIC_KEY, FORGED_PUBLIC_KEY,
-    UNSIGNED_SUBKEY, FORGED_SUBKEY, SUBKEY_FINGERPRINT,
-    UNEXPIRE_PUBLIC_KEY, OLD_UNEXPIRE_PUBLIC_KEY,
-    FORGED_UNEXPIRE_KEY, TWO_SIGNATURE_PUBLIC_KEYS, SECOND_KEY_FINGERPRINT,
-    SECOND_VALID_PUBLIC_KEY, TWO_KEYS_ONE_EXPIRED,
-    )
 from tests.test_recursiveloader import INSECURE_HASH_TESTS
 from tests.testutil import HKPServer
 
 
-MALFORMED_PUBLIC_KEY = b'''
------BEGIN PGP PUBLIC KEY BLOCK-----
+data_dir = Path(__file__).parent / "data"
 
-mQENBFnwXJMBCACgaTVz+d10TGL9zR920sb0GBFsitAJ5ZFzO4E0cg3SHhwI+reM
-JQ6LLKmHowY/E1dl5FBbnJoRMxXP7/eScQ7HlhYj1gMPN5XiS2pkPwVkmJKBDV42
-DLwoytC+ot0frRTJvSdEPCX81BNMgFiBSpkeZfXqb9XmU03bh6mFnrdd4CsHpTQG
-csVXHK8QKhaxuqmHTALdpSzKCb/r0N/Z3sQExZhfLcBf/9UUVXj44Nwc6ooqZLRi
-zHydxwQdxNu0aOFGEBn9WTi8Slf7MfR/pF0dI8rs9w6zMzVEq0lhDPpKFGDveoGf
-g/+TpvBNXZ7DWH23GM4kID3pk4LLMc24U1PhABEBAAG0D2dlbWF0byB0ZXN0IGtl
-eYkBRgQTAQoAMBYhBIHhLBa9jc1gvhgIRRNogOcqexOEBQJZ8FyTAhsDBQsJCg0E
-AxUKCAIeAQIXgAAKCRATaIDnKnsThCnkB/0fhTH230idhlfZhFbVgTLxrj4rpsGg
-20K8HkMaWzshsONdKkqYaYuRcm2UQZ0Kg5rm9jQsGYuAnzH/7XwmOleY95ycVfBk
-je9aXF6BEoGick6C/AK5w77vd1kcBtJDrT4I7vwD4wRkyUdCkpVMVT4z4aZ7lHJ4
-ECrrrI/mg0b+sGRyHfXPvIPp7F2959L/dpbhBZDfMOFC0A9LBQBJldKFbQLg3xzX
-4tniz/BBrp7KjTOMKU0sufsedI50xc6cvCYCwJElqo86vv69klZHahE/k9nJaUAM
-jCvJNJ7pU8YnJSRTQDH0PZEupAdzDU/AhGSrBz5+Jr7N0pQIxq4duE/Q
-=r7JK
------END PGP PUBLIC KEY BLOCK-----
-'''
 
-COMMON_MANIFEST_TEXT = """\
-TIMESTAMP 2017-10-22T18:06:41Z
-MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-IGNORE local
-DATA myebuild-0.ebuild 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709\
-"""
+def break_sig(sig):
+    """Return signature packet mangled to mismatch the signed key"""
+    return sig[:-1] + b'\x55'
 
-SIGNED_MANIFEST = f"""
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA256
 
-{COMMON_MANIFEST_TEXT}
------BEGIN PGP SIGNATURE-----
+def dash_escape(data: str) -> str:
+    """Add dash escapes to Manifest"""
+    return "".join(
+        [
+            f"- {x}\n"
+            if x.startswith(("TIMESTAMP", "MANIFEST", "DATA", "DIST"))
+            else f"{x}\n"
+            for x in data.splitlines()
+        ]
+    )
 
-iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAloCx+YACgkQE2iA5yp7
-E4TYrwf+JxjkVDNtvSN3HjQmdtcayLsaliw/2kqjoaQKs0lZD8+NRe7xPmwSm4bP
-XKfoouJ0+/s87vuYJpBBCjtUDA9C9yZIeRTo8+eW6XsZbRRUmUD5ylTS+FpSsUrS
-bEyYk4yZQMYrat+GQ1QBv+625nqnSDv5LZHBBZ/rG36GGlwHPbIKIishnDfdG2QQ
-zuxkqepNq4Inzp//ES7Bv4qbTzyBI//HzfY31vOgdhhs5N5Ytez3Xxv/KNOTYdi1
-ZIfqeaQ4NoefmxQunyEjT+8X2DMaEeHQni7dwjQc+FiN4ReV9aWbLo2O2cArqEHR
-mkkhTd2Auao4D2K74BePBuiZ9+eDQA==
-=khff
------END PGP SIGNATURE-----
-"""
 
-POST_EXPIRATION_SIGNED_MANIFEST = f"""
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA256
+def modify_manifest(data: str) -> str:
+    """Return the "modified" Manifest variation"""
+    return "".join(
+        [
+            x.split(" 0 ", 1)[0] + " 32\n"
+            if x.startswith("DATA")
+            else f"{x}\n"
+            for x in data.splitlines()
+        ]
+    )
 
-{COMMON_MANIFEST_TEXT}
------BEGIN PGP SIGNATURE-----
 
-iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAmPsj28ACgkQE2iA5yp7
-E4R0xAf8CC6uh8VMmv8xlFePEoBYEuSUtDa2hWHJv1sMn90QnszHGG6oo32g2Lje
-H9NRyjOltAG9t0siF/pf57EiKCs9B+Z9zLGYuWlK4gvkHjMHzsoTipUymm2/saEo
-AuoeZvhqNtfU0hCIJsWENtdyMb/hsJIxIOwBjVS/JT5cZlOGjhlyxVO0CS/7FsCp
-GZCeLYPdYXPw2em2DR3Q3NDuNmUY7W3WhJCL14uC+AkU64SnHc13xQ9/go6TQ2ho
-783Jm2f/4ZREYpKMvCgUJvOADSqnfY89hc6B/9JCXn+Zm8a31zgENlJ8DEhN0JMN
-le/JaXEH/AhO6xCOmk8tNQ3QXcNF5w==
-=UGgA
------END PGP SIGNATURE-----
-"""
+def _(path: str) -> bytes:
+    return data_dir.joinpath(path).read_bytes()
 
-DASH_ESCAPED_SIGNED_MANIFEST = '''
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA256
 
-- TIMESTAMP 2017-10-22T18:06:41Z
-- MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-IGNORE local
-- DATA myebuild-0.ebuild 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-- DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
------BEGIN PGP SIGNATURE-----
+def F(path: str) -> str:
+    return data_dir.joinpath(path).read_text().strip()
 
-iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAloCx+YACgkQE2iA5yp7
-E4TYrwf+JxjkVDNtvSN3HjQmdtcayLsaliw/2kqjoaQKs0lZD8+NRe7xPmwSm4bP
-XKfoouJ0+/s87vuYJpBBCjtUDA9C9yZIeRTo8+eW6XsZbRRUmUD5ylTS+FpSsUrS
-bEyYk4yZQMYrat+GQ1QBv+625nqnSDv5LZHBBZ/rG36GGlwHPbIKIishnDfdG2QQ
-zuxkqepNq4Inzp//ES7Bv4qbTzyBI//HzfY31vOgdhhs5N5Ytez3Xxv/KNOTYdi1
-ZIfqeaQ4NoefmxQunyEjT+8X2DMaEeHQni7dwjQc+FiN4ReV9aWbLo2O2cArqEHR
-mkkhTd2Auao4D2K74BePBuiZ9+eDQA==
-=khff
------END PGP SIGNATURE-----
-'''
 
-MODIFIED_SIGNED_MANIFEST = '''
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA256
+def T(path: str) -> str:
+    return data_dir.joinpath(path).read_text()
 
-TIMESTAMP 2017-10-22T18:06:41Z
-MANIFEST eclass/Manifest 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-IGNORE local
-DATA myebuild-0.ebuild 32
-MISC metadata.xml 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
-DIST mydistfile.tar.gz 0 MD5 d41d8cd98f00b204e9800998ecf8427e\
- SHA1 da39a3ee5e6b4b0d3255bfef95601890afd80709
------BEGIN PGP SIGNATURE-----
 
-iQEzBAEBCAAdFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAloCx+YACgkQE2iA5yp7
-E4TYrwf+JxjkVDNtvSN3HjQmdtcayLsaliw/2kqjoaQKs0lZD8+NRe7xPmwSm4bP
-XKfoouJ0+/s87vuYJpBBCjtUDA9C9yZIeRTo8+eW6XsZbRRUmUD5ylTS+FpSsUrS
-bEyYk4yZQMYrat+GQ1QBv+625nqnSDv5LZHBBZ/rG36GGlwHPbIKIishnDfdG2QQ
-zuxkqepNq4Inzp//ES7Bv4qbTzyBI//HzfY31vOgdhhs5N5Ytez3Xxv/KNOTYdi1
-ZIfqeaQ4NoefmxQunyEjT+8X2DMaEeHQni7dwjQc+FiN4ReV9aWbLo2O2cArqEHR
-mkkhTd2Auao4D2K74BePBuiZ9+eDQA==
-=khff
------END PGP SIGNATURE-----
-'''
+# == first key (used for most of the tests) ==
+PUBLIC_KEY = _("first-key/pub")
+SECRET_KEY = _("first-key/secret")
+# (subkey optionally used)
+PUBLIC_SUBKEY = _("first-key/subkey")
+# main UID
+UID = _("first-key/uid")
+# UID without email
+UID_NOEMAIL = _("first-key/uid-noemail")
+# UID that's not valid UTF-8
+UID_NONUTF = _("first-key/uid-nonutf")
 
-EXPIRED_SIGNED_MANIFEST = f"""
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA256
+# UID signatures without expiration date
+PUBLIC_KEY_SIG = _("first-key/uid-sig")
+PUBLIC_KEY_NOEMAIL_SIG = _("first-key/uid-noemail-sig")
+PUBLIC_KEY_NONUTF_SIG = _("first-key/uid-nonutf-sig")
+# subkey signature without expiration date
+PUBLIC_SUBKEY_SIG = _("first-key/subkey-sig")
+# main UID signature that expired
+EXPIRED_KEY_SIG = _("first-key/expired-sig")
+# main UID revocation signature
+REVOCATION_SIG = _("first-key/revocation-sig")
+# main UID signature without expiration that's newer than expired-sig
+UNEXPIRE_SIG = _("first-key/unexpire-sig")
 
-{COMMON_MANIFEST_TEXT}
------BEGIN PGP SIGNATURE-----
+# == other key (used in WKD tests, has the same UID as first key) ==
+OTHER_PUBLIC_KEY = _("other-key/pub")
+OTHER_PUBLIC_KEY_UID = _("other-key/uid")
+OTHER_PUBLIC_KEY_SIG = _("other-key/uid-sig")
 
-iQE5BAEBCAAjFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAlnxCXcFgwABUYAACgkQ
-E2iA5yp7E4SDpQgAizTfQ6HJ1mawgElYV1LsOKGT8ivC6CAeU3Cs1E8zYpitKuy7
-Yu5WrqUgck5GkXfswxHISkV+oWzrA/j0bUV768o+fY2JmlKuc/VWeyYDGnDtgDPz
-NXYoqlQ1z3TDeaRktHcblECghf/A9Hbw0L4i0DVvDdk9APtIswgL/RmpXAQS1Bl7
-sE1aFIy8CMBf3itco7NGjPpCxRt7ckS+UIKNgzrfnS7WEXHIirykEsMYKTLfuN2u
-HSxRUCkTK1jBuP/v/rjdqUJw3LXAbjxFl9SyUX4AgCgHqgso3IZwjAprQRKNSObO
-t5pTRGhLWgdLUrs7vRB7wf7F8h4sci/YBKJRFA==
-=VGMV
------END PGP SIGNATURE-----
-"""
+# == second key (using different UID) ==
+SECOND_PUBLIC_KEY = _("second-key/pub")
+SECOND_SECRET_KEY = _("second-key/secret")
+SECOND_UID = _("second-key/uid")
+SECOND_KEY_SIG = _("second-key/uid-sig")
 
-SUBKEY_SIGNED_MANIFEST = f"""
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA256
+# == combined variants of first-key ==
+VALID_PUBLIC_KEY = PUBLIC_KEY + UID + PUBLIC_KEY_SIG
+EXPIRED_PUBLIC_KEY = PUBLIC_KEY + UID + PUBLIC_KEY_SIG + EXPIRED_KEY_SIG
+REVOKED_PUBLIC_KEY = PUBLIC_KEY + REVOCATION_SIG + UID + PUBLIC_KEY_SIG
+# using the original signature to "unexpire" a key should fail
+OLD_UNEXPIRE_PUBLIC_KEY = PUBLIC_KEY + UID + PUBLIC_KEY_SIG
+UNEXPIRE_PUBLIC_KEY = PUBLIC_KEY + UID + UNEXPIRE_SIG
 
-{COMMON_MANIFEST_TEXT}
------BEGIN PGP SIGNATURE-----
+PRIVATE_KEY = SECRET_KEY + UID + PUBLIC_KEY_SIG
 
-iLMEAQEIAB0WIQR+nd48vkfkN0GN90A4udL3bMgzzAUCX0UGrAAKCRA4udL3bMgz
-zH8MA/93/oNkXaA8+ZX7s8umhNMHiovdLJMna7Bl2C/tEdLfOoyp9o3lChhnB49v
-g7VRUc//lz5sDUShdUUlTYjCPGLaYf2rBZHqd5POGJOsbzu1Tmtd8uhWFWnl8Kip
-n4XmpdPvu+UdAHpQIGzKoNOEDJpZ5CzPLhYa5KgZiJhpYsDXgg==
-=lpJi
------END PGP SIGNATURE-----
-"""
+VALID_KEY_NOEMAIL = PUBLIC_KEY + UID_NOEMAIL + PUBLIC_KEY_NOEMAIL_SIG
+VALID_KEY_NONUTF = PUBLIC_KEY + UID_NONUTF + PUBLIC_KEY_NONUTF_SIG
 
-TWO_SIGNATURES = """
-iQFHBAABCAAxFiEEgeEsFr2NzWC+GAhFE2iA5yp7E4QFAmPMHYQTHGdlbWF0b0Bl
-eGFtcGxlLmNvbQAKCRATaIDnKnsThCDWB/95B9njv423M94uRdpPqSNqTpAokNhy
-V0hjnhpiqnY85iFdL1Zc/rvhuxYbZezrig3dqctLseWYcx2mINBTLZqWHk5/NKEm
-rd8iCdXZU1B7yo/HCfzUYR4HX5wISCiRjKimFFgkWKOg7KYGOqqrwLjAjaYJKmL5
-L7R5joHpGbp87jix7c0ruSIMslQg5PbJ6/YAQWyOPTcZvqMFieJ8tqE/G2FabQcs
-YRHEGu1x8wNY40rFzWd90ICR/hPjXZlCdCN2qk7hs+Coasb29n6pXjmt5L8/ICcL
-zApRg8cetid6/SIzUSwiVqBt7i8noYWbgaazNt3HDlGq55v21dkOhmrXiIkEABYI
-ADEWIQR1jj6cjPscaH2bJCVTcI9ps0i0zAUCY8wd6BMcc2Vjb25kQGV4YW1wbGUu
-Y29tAAoJEFNwj2mzSLTMHKcA/0QbVl3PafYp45PFFo2e/knGKJKrm8D4bUH9wS5h
-dchVAP0RSzkUQPP7Zs+2uHQItkqbXJyrBBHOqjGzeh39sWVuAw==
-"""
+VALID_KEY_SUBKEY = (PUBLIC_KEY + UID + PUBLIC_KEY_SIG + PUBLIC_SUBKEY +
+                    PUBLIC_SUBKEY_SIG)
 
-TWO_SIGNATURE_MANIFEST = f"""
------BEGIN PGP SIGNED MESSAGE-----
-Hash: SHA256
+# first-key with broken signature
+FORGED_PUBLIC_KEY = PUBLIC_KEY + UID + break_sig(PUBLIC_KEY_SIG)
+FORGED_SUBKEY = (PUBLIC_KEY + UID + PUBLIC_KEY_SIG + PUBLIC_SUBKEY +
+                 break_sig(PUBLIC_SUBKEY_SIG))
+FORGED_UNEXPIRE_KEY = (PUBLIC_KEY + UID + EXPIRED_KEY_SIG +
+                       break_sig(UNEXPIRE_SIG))
 
-{COMMON_MANIFEST_TEXT}
------BEGIN PGP SIGNATURE-----
+# first-key without signatures
+UNSIGNED_PUBLIC_KEY = PUBLIC_KEY + UID
+UNSIGNED_SUBKEY = PUBLIC_KEY + UID + PUBLIC_KEY_SIG + PUBLIC_SUBKEY
 
-{TWO_SIGNATURES}
-=wG4b
------END PGP SIGNATURE-----
-"""
+# == key fingerprints ==
+KEY_FINGERPRINT = F("first-key/fpr.txt")
+SUBKEY_FINGERPRINT = F("first-key/sub-fpr.txt")
+OTHER_KEY_FINGERPRINT = F("other-key/fpr.txt")
+SECOND_KEY_FINGERPRINT = F("second-key/fpr.txt")
+
+# == combined variants for other-key ==
+OTHER_VALID_PUBLIC_KEY = (OTHER_PUBLIC_KEY + OTHER_PUBLIC_KEY_UID +
+                          OTHER_PUBLIC_KEY_SIG)
+COMBINED_PUBLIC_KEYS = OTHER_VALID_PUBLIC_KEY + VALID_PUBLIC_KEY
+
+# == combined variants for second-key ==
+SECOND_VALID_PUBLIC_KEY = SECOND_PUBLIC_KEY + SECOND_UID + SECOND_KEY_SIG
+TWO_SIGNATURE_PUBLIC_KEYS = VALID_PUBLIC_KEY + SECOND_VALID_PUBLIC_KEY
+TWO_KEYS_ONE_EXPIRED = EXPIRED_PUBLIC_KEY + SECOND_VALID_PUBLIC_KEY
+
+# key with CRC error
+MALFORMED_PUBLIC_KEY = _("malformed-key.txt")
+# Manifest signed before first-key expired
+SIGNED_MANIFEST = T("Manifest.asc")
+# Manifest signed after first-key expired
+POST_EXPIRATION_SIGNED_MANIFEST = T("Manifest.asc-post-expiration")
+# valid Manifest with dash-escaped content
+DASH_ESCAPED_SIGNED_MANIFEST = dash_escape(SIGNED_MANIFEST)
+# Manifest with modified text (should fail)
+MODIFIED_SIGNED_MANIFEST = modify_manifest(SIGNED_MANIFEST)
+# Manifest with expired signature itself
+EXPIRED_SIGNED_MANIFEST = T("Manifest.asc-expired")
+# Manifest signed using the subkey
+SUBKEY_SIGNED_MANIFEST = T("Manifest.asc-subkey-signed")
+# combined signatures from first-key + second-key
+TWO_SIGNATURE_MANIFEST = T("Manifest.asc-two-signatures")
+
+# expected / base Manifest path (used for detached signature tests)
+MANIFEST_PATH = data_dir / "Manifest"
+# combined signatures from first-key + second-key
+TWO_SIGNATURE_PATH = data_dir / "two-signatures.bin"
 
 
 def strip_openpgp(text):
     lines = text.lstrip().splitlines()
     start = lines.index('')
     stop = lines.index('-----BEGIN PGP SIGNATURE-----')
-    return '\n'.join(lines[start+1:stop-start+2]) + '\n'
+    return '\n'.join(lines[start+1:stop-start+1]) + '\n'
 
 
-# workaround pyflakes' limitations
-_ = COMBINED_PUBLIC_KEYS
-_ = EXPIRED_PUBLIC_KEY
-_ = FORGED_PUBLIC_KEY
-_ = FORGED_SUBKEY
-_ = FORGED_UNEXPIRE_KEY
-_ = OLD_UNEXPIRE_PUBLIC_KEY
-_ = OTHER_VALID_PUBLIC_KEY
-_ = SECOND_VALID_PUBLIC_KEY
-_ = TWO_KEYS_ONE_EXPIRED
-_ = TWO_SIGNATURE_PUBLIC_KEYS
-_ = UNEXPIRE_PUBLIC_KEY
-_ = UNSIGNED_PUBLIC_KEY
-_ = UNSIGNED_SUBKEY
-_ = VALID_KEY_NOEMAIL
-_ = VALID_KEY_NONUTF
-_ = VALID_KEY_SUBKEY
+@cache
+def is_sequoia() -> bool:
+    """Return True if GNUPG is sequoia-chameleon-gnupg"""
+    try:
+        out = subprocess.run([GNUPG, "--version"], capture_output=True)
+    except OSError:
+        return False
+    return out.returncode == 0 and b"sequoia" in out.stdout
+
+
+sequoia_xfail = pytest.mark.xfail(is_sequoia(),
+                                  reason="FIXME on sequoia-chameleon-gnupg")
 
 
 @pytest.mark.parametrize('manifest_var',
@@ -452,7 +407,7 @@ def assert_signature(sig: OpenPGPSignatureList,
         assert sorted(sig) == sorted([
             OpenPGPSignatureData(
                 fingerprint=KEY_FINGERPRINT,
-                timestamp=datetime.datetime(2023, 1, 21, 17, 14, 44),
+                timestamp=datetime.datetime(2024, 1, 1),
                 primary_key_fingerprint=KEY_FINGERPRINT,
                 sig_status=OpenPGPSignatureStatus.GOOD,
                 trusted_sig=True,
@@ -460,7 +415,7 @@ def assert_signature(sig: OpenPGPSignatureList,
                 ),
             OpenPGPSignatureData(
                 fingerprint=SECOND_KEY_FINGERPRINT,
-                timestamp=datetime.datetime(2023, 1, 21, 17, 16, 24),
+                timestamp=datetime.datetime(2020, 1, 1),
                 primary_key_fingerprint=SECOND_KEY_FINGERPRINT,
                 sig_status=OpenPGPSignatureStatus.GOOD,
                 trusted_sig=True,
@@ -470,19 +425,19 @@ def assert_signature(sig: OpenPGPSignatureList,
     elif manifest_var == 'SUBKEY_SIGNED_MANIFEST':
         assert len(sig) == 1
         assert sig.fingerprint == SUBKEY_FINGERPRINT
-        assert sig.timestamp == datetime.datetime(2020, 8, 25, 12, 40, 12)
+        assert sig.timestamp == datetime.datetime(2020, 1, 1)
         assert sig.expire_timestamp is None
         assert sig.primary_key_fingerprint == KEY_FINGERPRINT
     elif manifest_var == "POST_EXPIRATION_SIGNED_MANIFEST":
         assert len(sig) == 1
         assert sig.fingerprint == KEY_FINGERPRINT
-        assert sig.timestamp == datetime.datetime(2023, 2, 15, 7, 53, 19)
+        assert sig.timestamp == datetime.datetime(2024, 1, 1)
         assert sig.expire_timestamp is None
         assert sig.primary_key_fingerprint == KEY_FINGERPRINT
     else:
         assert len(sig) == 1
         assert sig.fingerprint == KEY_FINGERPRINT
-        assert sig.timestamp == datetime.datetime(2017, 11, 8, 9, 1, 26)
+        assert sig.timestamp == datetime.datetime(2020, 1, 1)
         assert sig.expire_timestamp is None
         assert sig.primary_key_fingerprint == KEY_FINGERPRINT
 
@@ -718,7 +673,8 @@ def privkey_env(request):
 TEST_STRING = 'The quick brown fox jumps over the lazy dog'
 
 
-@pytest.mark.parametrize('keyid', [None, PRIVATE_KEY_ID])
+@sequoia_xfail
+@pytest.mark.parametrize('keyid', [None, KEY_FINGERPRINT])
 def test_sign_data(privkey_env, keyid):
     """Test signing data"""
     with io.StringIO(TEST_STRING) as f:
@@ -728,10 +684,13 @@ def test_sign_data(privkey_env, keyid):
             privkey_env.verify_file(wf)
 
 
-@pytest.mark.parametrize('keyid', [None, PRIVATE_KEY_ID])
+@pytest.mark.parametrize('keyid', [None, KEY_FINGERPRINT])
 @pytest.mark.parametrize('sign', [None, False, True])
 def test_dump_signed_manifest(privkey_env, keyid, sign):
     """Test dumping a signed Manifest"""
+    if sign is not False and is_sequoia():
+        pytest.xfail("FIXME: sequoia-chameleon-gnupg fails to sign")
+
     m = ManifestFile()
     verify = True if sign is None else False
     with io.StringIO(SIGNED_MANIFEST) as f:
@@ -751,6 +710,7 @@ def test_dump_signed_manifest(privkey_env, keyid, sign):
         assert m.openpgp_signature is None
 
 
+@sequoia_xfail
 @pytest.mark.parametrize('filename', ['Manifest', 'Manifest.gz'])
 @pytest.mark.parametrize('sign', [None, True])
 def test_recursive_manifest_loader_save_manifest(tmp_path, privkey_env,
@@ -801,19 +761,42 @@ def test_recursive_manifest_loader_save_submanifest(tmp_path, privkey_env):
 
 @pytest.mark.parametrize(
     'key_var,expected',
-    [('VALID_PUBLIC_KEY', {KEY_FINGERPRINT: ['gemato@example.com']}),
+    [('VALID_PUBLIC_KEY',
+      {KEY_FINGERPRINT: [b"gemato test key <gemato@example.com>"]}),
      ('OTHER_VALID_PUBLIC_KEY',
-      {OTHER_KEY_FINGERPRINT: ['gemato@example.com']}),
-     ('VALID_KEY_SUBKEY', {KEY_FINGERPRINT: ['gemato@example.com']}),
-     ('VALID_KEY_NOEMAIL', {KEY_FINGERPRINT: []}),
-     ('VALID_KEY_NONUTF', {KEY_FINGERPRINT: ['gemato@example.com']}),
+      {OTHER_KEY_FINGERPRINT: [b"gemato test key <gemato@example.com>"]}),
+     ('VALID_KEY_SUBKEY',
+      {KEY_FINGERPRINT: [b"gemato test key <gemato@example.com>"]}),
+     ('VALID_KEY_NOEMAIL',
+      {KEY_FINGERPRINT: [b"gemato test key"]}),
+     ('VALID_KEY_NONUTF',
+      {KEY_FINGERPRINT: [
+           b"gemat\\xf6 test key <gemato@example.com>" if is_sequoia()
+           else b"gemat\xf6 test key <gemato@example.com>"
+       ]}),
      ])
-def test_list_keys(openpgp_env_with_refresh, key_var, expected):
+def test_list_keys(openpgp_env, key_var, expected):
     try:
-        openpgp_env_with_refresh.import_key(io.BytesIO(globals()[key_var]))
+        openpgp_env.import_key(io.BytesIO(globals()[key_var]))
+    except OpenPGPNoImplementation as e:
+        pytest.skip(str(e))
+    assert openpgp_env.list_keys() == expected
+    assert openpgp_env.list_keys(list(expected.keys())) == expected
+
+
+def test_list_keys_empty(openpgp_env):
+    try:
+        assert openpgp_env.list_keys() == {}
+    except OpenPGPNoImplementation as e:
+        pytest.skip(str(e))
+
+
+def test_list_keys_error(openpgp_env):
+    try:
+        with pytest.raises(OpenPGPKeyListingError):
+            openpgp_env.list_keys(["zzzzz"])
     except OpenPGPNoImplementation as e:
         pytest.skip(str(e))
-    assert openpgp_env_with_refresh.list_keys() == expected
 
 
 @pytest.fixture(scope='module')
@@ -1005,8 +988,8 @@ def signal_desc(sig):
     'command,expected,match',
     [('true', 0, None),
      ('false', 1, None),
-     ('gpg --verify {tmp_path}/Manifest', 0, None),
-     ('gpg --verify {tmp_path}/Manifest.subkey', 2, None),
+     ('{gpg} --verify {tmp_path}/Manifest', 0, None),
+     ('{gpg} --verify {tmp_path}/Manifest.subkey', 2, None),
      ('sh -c "kill $$"', -signal.SIGTERM,
       f'Child process terminated due to signal: '
       f'{signal_desc(signal.SIGTERM)}'),
@@ -1022,7 +1005,7 @@ def test_cli_gpg_wrap(tmp_path, caplog, command, expected, match):
     with open(tmp_path / 'Manifest.subkey', 'w') as f:
         f.write(SUBKEY_SIGNED_MANIFEST)
 
-    command = [x.replace('{tmp_path}', str(tmp_path))
+    command = [x.replace('{tmp_path}', str(tmp_path)).replace('{gpg}', GNUPG)
                for x in shlex.split(command)]
     retval = gemato.cli.main(['gemato', 'gpg-wrap',
                               '--openpgp-key',
@@ -1078,6 +1061,12 @@ def test_recursive_manifest_loader_require_secure(tmp_path, privkey_env,
      ])
 def test_update_require_secure_cli(base_tree, caplog, hashes_arg,
                                    insecure, sign, require_secure):
+    expected = (1 if insecure is not None and sign != "--no-sign"
+                and require_secure != "--no-require-secure-hashes"
+                else 0)
+    if expected == 0 and is_sequoia():
+        pytest.xfail("FIXME: sequoia-chameleon-gnupg fails to sign")
+
     with open(base_tree / ".key.bin", "wb") as keyf:
         keyf.write(PRIVATE_KEY)
     with open(base_tree / "Manifest", "w") as f:
@@ -1091,9 +1080,6 @@ def test_update_require_secure_cli(base_tree, caplog, hashes_arg,
     if str(OpenPGPNoImplementation('install gpg')) in caplog.text:
         pytest.skip('OpenPGP implementation missing')
 
-    expected = (1 if insecure is not None and sign != "--no-sign"
-                and require_secure != "--no-require-secure-hashes"
-                else 0)
     assert retval == expected
     if expected == 1:
         assert str(ManifestInsecureHashes(insecure)) in caplog.text
@@ -1134,15 +1120,9 @@ def test_verify_detached(tmp_path, key_var, two_sigs):
             with io.BytesIO(globals()[key_var]) as f:
                 openpgp_env.import_key(f)
 
-            with open(tmp_path / "data.bin", "wb") as f:
-                f.write(b"\r\n".join(COMMON_MANIFEST_TEXT.encode("utf8")
-                                     .splitlines()))
-            with open(tmp_path / "sig.bin", "wb") as f:
-                f.write(base64.b64decode(TWO_SIGNATURES))
-
-            with open(tmp_path / "data.bin", "rb") as f:
+            with open(MANIFEST_PATH, "rb") as f:
                 sig = openpgp_env.verify_detached(
-                    tmp_path / "sig.bin", f,
+                    TWO_SIGNATURE_PATH, f,
                     require_all_good=two_sigs)
 
             assert_signature(sig, "TWO_SIGNATURE_MANIFEST",
